'''Copyright The Cambricon Team'''

import os
import pkgutil
import importlib

from .abstract_accelerator import DeepSpeedAccelerator
try:
    import torch
    import torch_mlu
except ImportError:
    pass

class MLU_Accelerator(DeepSpeedAccelerator):
    def __init__(self):
        self._name = 'mlu'
        self._communication_backend_name = 'cncl'
        
    def is_synchronized_device(self):
        return False

    # Device APIs
    def device_name(self, device_index=None):
        if device_index == None:
            return 'mlu'
        return 'mlu:{}'.format(device_index)

    def device(self, device_index=None):
        return torch.mlu.device(device_index)

    def set_device(self, device_index):
        torch.mlu.set_device(device_index)

    def current_device(self):
        return torch.mlu.current_device()

    def current_device_name(self):
        return 'mlu:{}'.format(torch.mlu.current_device())

    def device_count(self):
        return torch.mlu.device_count()

    def synchronize(self, device_index=None):
        return torch.mlu.synchronize(device_index)

    # RNG APIs
    def random(self):
        return torch.random

    def set_rng_state(self, new_state, device_index=None):
        if device_index is None:
            return torch.mlu.set_rng_state(new_state)

        return torch.mlu.set_rng_state(new_state, device_index)

    def get_rng_state(self, device_index=None):
        if device_index is None:
            return torch.mlu.get_rng_state()

        return torch.mlu.get_rng_state(device_index)

    def manual_seed(self, seed):
        return torch.mlu.manual_seed(seed)

    def manual_seed_all(self, seed):
        return torch.mlu.manual_seed_all(seed)

    def initial_seed(self, seed):
        return torch.mlu.initial_seed(seed)

    def default_generator(self, device_index):
        return torch.mlu.default_generators[device_index]

    # Streams/Events
    @property
    def Stream(self):
        return torch.mlu.Stream

    def stream(self, stream):
        return torch.mlu.stream(stream)

    def current_stream(self, device_index=None):
        return torch.mlu.current_stream(device_index)

    def default_stream(self, device_index=None):
        return torch.mlu.default_stream(device_index)

    @property
    def Event(self):
        return torch.mlu.Event

    # Memory management
    def empty_cache(self):
        return torch.mlu.empty_cache()

    def memory_allocated(self, device_index=None):
        return torch.mlu.memory_allocated(device_index)

    def max_memory_allocated(self, device_index=None):
        return torch.mlu.max_memory_allocated(device_index)

    def reset_max_memory_allocated(self, device_index=None):
        return torch.mlu.reset_max_memory_allocated(device_index)

    def memory_cached(self, device_index=None):
        return torch.mlu.memory_cached(device_index)

    def max_memory_cached(self, device_index=None):
        return torch.mlu.max_memory_cached(device_index)

    def reset_max_memory_cached(self, device_index=None):
        return torch.mlu.reset_max_memory_cached(device_index)

    def memory_stats(self, device_index=None):
        if hasattr(torch.mlu, 'memory_stats'):
            return torch.mlu.memory_stats(device_index)

    def reset_peak_memory_stats(self, device_index=None):
        if hasattr(torch.mlu, 'reset_peak_memory_stats'):
            return torch.mlu.reset_peak_memory_stats(device_index)

    def memory_reserved(self, device_index=None):
        if hasattr(torch.mlu, 'memory_reserved'):
            return torch.mlu.memory_reserved(device_index)

    def max_memory_reserved(self, device_index=None):
        if hasattr(torch.mlu, 'max_memory_reserved'):
            return torch.mlu.max_memory_reserved(device_index)

    def total_memory(self, device_index=None):
        return torch.mlu.get_device_properties(device_index).total_memory

    # Data types
    def is_bf16_supported(self):
        return torch.mlu.is_bf16_supported()

    def is_fp16_supported(self):
        major, _ = torch.mlu.get_device_capability()
        if major >= 7:
            return True
        else:
            return False
        
    def supported_dtypes(self):
        return [torch.float, torch.half]

    # Misc
    def amp(self):
        if hasattr(torch.mlu, 'amp'):
            return torch.mlu.amp
        return None

    def is_available(self):
        return torch.mlu.is_available()

    def range_push(self, msg):
        if hasattr(torch.mlu, "nvtx") and hasattr(torch.mlu.nvtx, 'range_push'):
            return torch.mlu.nvtx.range_push(msg)

    def range_pop(self):
        if hasattr(torch.mlu, "nvtx") and hasattr(torch.mlu.nvtx, 'range_pop'):
            return torch.mlu.nvtx.range_pop()

    def lazy_call(self, callback):
        return torch.mlu._lazy_call(callback)

    def communication_backend_name(self):
        return self._communication_backend_name

    # Tensor operations

    @property
    def BFloat16Tensor(self):
        return torch.mlu.BFloat16Tensor

    @property
    def ByteTensor(self):
        return torch.mlu.ByteTensor

    @property
    def DoubleTensor(self):
        return torch.mlu.DoubleTensor

    @property
    def FloatTensor(self):
        return torch.mlu.FloatTensor

    @property
    def HalfTensor(self):
        return torch.mlu.HalfTensor

    @property
    def IntTensor(self):
        return torch.mlu.IntTensor

    @property
    def LongTensor(self):
        return torch.mlu.LongTensor

    def pin_memory(self, tensor):
        return tensor.pin_memory()

    def on_accelerator(self, tensor):
        device_str = str(tensor.device)
        if device_str.startswith('mlu:'):
            return True
        else:
            return False

    def op_builder_dir(self):
        try:
            # is op_builder from deepspeed or a 3p version? this should only succeed if it's deepspeed
            # if successful this also means we're doing a local install and not JIT compile path
            from op_builder import __deepspeed__  # noqa: F401
            return "op_builder.mlu"
        except ImportError:
            return "deepspeed.ops.op_builder.mlu"

    # create an instance of op builder and return, name specified by class_name
    def create_op_builder(self, class_name):
        builder_class = self.get_op_builder(class_name)
        if builder_class != None:
            return builder_class()
        return None


    # return an op builder class, name specified by class_name
    def get_op_builder(self, class_name):
        try:
            # is op_builder from deepspeed or a 3p version? this should only succeed if it's deepspeed
            # if successful this also means we're doing a local install and not JIT compile path
            from op_builder import __deepspeed__  # noqa: F401 # type: ignore
            from op_builder.mlu import CPUAdamBuilder, NotImplementedBuilder
        except ImportError:
            from deepspeed.ops.op_builder.mlu import CPUAdamBuilder, NotImplementedBuilder

        if class_name == "CPUAdamBuilder":
            return CPUAdamBuilder
        else:
            # return a NotImplementedBuilder to avoid get NoneType[Name] in unit tests
            return NotImplementedBuilder

    def build_extension(self):
        from torch.utils.cpp_extension import BuildExtension
        return BuildExtension
